@@ -32,8 +32,10 @@ public:
     QRegExp imageRegex;
 };
 
+static const KCatalogLoader loader(QLatin1String("ktp-filters"));
+
 ImagesFilter::ImagesFilter (QObject* parent, const QVariantList&) :
-    AbstractMessageFilter (parent), d(new Private)
+    KTp::AbstractMessageFilter (parent), d(new Private)
 {
     QString imagePattern = QLatin1String("\\.(?:");
     Q_FOREACH (const QByteArray &format, QImageReader::supportedImageFormats()) {
@@ -50,8 +52,9 @@ ImagesFilter::~ImagesFilter()
     delete d;
 }
 
-void ImagesFilter::filterMessage (Message& message)
+void ImagesFilter::filterMessage(KTp::Message &message, const KTp::MessageContext &context)
 {
+    Q_UNUSED(context)
     kDebug() << message.property("Urls").toList().size();
     Q_FOREACH (const QVariant &var, message.property("Urls").toList()) {
         const KUrl url = qvariant_cast<KUrl>(var);
