@@ -26,6 +26,8 @@
 #include "chat-window-style-manager.h"
 #include "chat-window-style.h"
 
+#include <KTp/message-processor.h>
+
 #include <QtCore/QFile>
 #include <QtCore/QTextCodec>
 #include <QtGui/QContextMenuEvent>
@@ -66,6 +68,9 @@ AdiumThemeView::AdiumThemeView(QWidget *parent)
             this, SLOT(onOpenLinkActionTriggered()));
 
     connect(this, SIGNAL(linkClicked(QUrl)), this, SLOT(onLinkClicked(QUrl)));
+
+    QWebSettings *ws = settings();
+    ws->setAttribute(QWebSettings::ZoomTextOnly, true);
 }
 
 void AdiumThemeView::load(ChatType chatType) {
@@ -111,6 +116,8 @@ void AdiumThemeView::load(ChatType chatType) {
     m_useCustomFont = appearanceConfig.readEntry("useCustomFont", false);
     m_fontFamily = appearanceConfig.readEntry("fontFamily", QWebSettings::globalSettings()->fontFamily(QWebSettings::StandardFont));
     m_fontSize = appearanceConfig.readEntry("fontSize", QWebSettings::globalSettings()->fontSize(QWebSettings::DefaultFontSize));
+
+    m_showPresenceMode = static_cast<AdiumThemeView::ShowPresenceMode>(appearanceConfig.readEntry("showPresenceMode", (int) AdiumThemeView::Always));
 }
 
 
@@ -131,6 +138,26 @@ void AdiumThemeView::contextMenuEvent(QContextMenuEvent *event)
     }
 }
 
+void AdiumThemeView::wheelEvent(QWheelEvent* event)
+{
+    // Zoom text on Ctrl + Scroll
+    if (event->modifiers() & Qt::CTRL) {
+        qreal factor = zoomFactor();
+        if (event->delta() > 0) {
+            factor += 0.1;
+        } else if (event->delta() < 0) {
+            factor -= 0.1;
+        }
+        setZoomFactor(factor);
+        Q_EMIT zoomFactorChanged(factor);
+
+        event->accept();
+        return;
+    }
+
+    QWebView::wheelEvent(event);
+}
+
 void AdiumThemeView::initialise(const AdiumThemeHeaderInfo &chatInfo)
 {
     QString headerHtml;
@@ -211,6 +238,12 @@ void AdiumThemeView::initialise(const AdiumThemeHeaderInfo &chatInfo)
     index = templateHtml.indexOf(QLatin1String("%@"), index);
     templateHtml.replace(index, 2, footerHtml);
 
+    // Inject the scripts and the css just before the end of the head tag
+    index = templateHtml.indexOf(QLatin1String("</head>"));
+    templateHtml.insert(index, KTp::MessageProcessor::instance()->header());
+
+    //kDebug() << templateHtml;
+
     setHtml(templateHtml);
 
     //hidden HTML debugging mode. Should have no visible way to turn it on.
@@ -283,6 +316,17 @@ bool AdiumThemeView::isCustomFont() const
     return m_useCustomFont;
 }
 
+void AdiumThemeView::setShowPresenceMode(ShowPresenceMode showPresenceMode)
+{
+    kDebug();
+    m_showPresenceMode = showPresenceMode;
+}
+
+AdiumThemeView::ShowPresenceMode AdiumThemeView::showPresenceMode() const
+{
+    return m_showPresenceMode;
+}
+
 bool AdiumThemeView::isHeaderDisplayed() const
 {
     return m_displayHeader;
@@ -310,8 +354,14 @@ void AdiumThemeView::addContentMessage(const AdiumThemeContentInfo &contentMessa
     // contentMessage is const, we need a non-const one to append message classes
     AdiumThemeContentInfo message(contentMessage);
 
+    // 2 consecutive messages can be combined when:
+    //  * Sender is the same
+    //  * Message type is the same
+    //  * Both have the "mention" class, or none of them have it
+    //  * Theme does not disable consecutive messages
     if (m_lastContent.senderScreenName() == message.senderScreenName()
         && m_lastContent.type() == message.type()
+        && m_lastContent.messageClasses().contains(QLatin1String("mention")) == message.messageClasses().contains(QLatin1String("mention"))
         && !m_chatStyle->disableCombineConsecutive()) {
         consecutiveMessage = true;
         message.appendMessageClass(QLatin1String("consecutive"));
@@ -359,7 +409,7 @@ void AdiumThemeView::addContentMessage(const AdiumThemeContentInfo &contentMessa
                                  willAddMoreContentObjects,
                                  replaceLastContent);
 
-    appendMessage(styleHtml, mode);
+    appendMessage(styleHtml, message.script(), mode);
 }
 
 void AdiumThemeView::addStatusMessage(const AdiumThemeStatusInfo& statusMessage)
@@ -397,7 +447,7 @@ void AdiumThemeView::addStatusMessage(const AdiumThemeStatusInfo& statusMessage)
                                  willAddMoreContentObjects,
                                  replaceLastContent);
 
-    appendMessage(styleHtml, mode);
+    appendMessage(styleHtml, message.script(), mode);
 }
 
 QString AdiumThemeView::appendScript(AdiumThemeView::AppendMode mode)
@@ -468,11 +518,15 @@ AdiumThemeView::AppendMode AdiumThemeView::appendMode(const AdiumThemeMessageInf
     return mode;
 }
 
-void AdiumThemeView::appendMessage(QString &html, AppendMode mode)
+void AdiumThemeView::appendMessage(QString &html, const QString &script, AppendMode mode)
 {
     QString js = appendScript(mode).arg(html.replace(QLatin1Char('\"'), QLatin1String("\\\""))
                                             .replace(QLatin1Char('\n'), QLatin1String("")));
     page()->mainFrame()->evaluateJavaScript(js);
+
+    if (!script.isEmpty()) {
+        page()->mainFrame()->evaluateJavaScript(script);
+    }
 }
 
 void AdiumThemeView::onLinkClicked(const QUrl &url)
@@ -566,7 +620,6 @@ QString AdiumThemeView::replaceMessageKeywords(QString &htmlTemplate, const Adiu
 
     //messageDirection
     htmlTemplate.replace(QLatin1String("%messageDirection%"), info.messageDirection());
-    
     htmlTemplate.replace(QLatin1String("%messageClasses%"), info.messageClasses());
 
 
@@ -648,6 +701,3 @@ const QString AdiumThemeView::variantPath() const
 {
     return m_variantPath;
 }
-
-
-
