@@ -59,6 +59,12 @@ public:
         ReplaceLastMessage
     };
 
+    enum ShowPresenceMode {
+        Always = 0,
+        NeverInGroupChats = 1,
+        Never = 2
+    };
+
     explicit AdiumThemeView(QWidget *parent = 0);
 
     /** Loads the Theme data*/
@@ -86,6 +92,9 @@ public:
     void setHeaderDisplayed(bool);
     /* .. font, backgrounds, everything else.*/
 
+    void setShowPresenceMode(ShowPresenceMode showPresenceMode);
+    ShowPresenceMode showPresenceMode() const;
+
     void clear();
 
 public Q_SLOTS:
@@ -96,6 +105,10 @@ public Q_SLOTS:
 
 protected:
     virtual void contextMenuEvent(QContextMenuEvent *event);
+    virtual void wheelEvent(QWheelEvent *event);
+
+Q_SIGNALS:
+    void zoomFactorChanged(qreal zoomFactor);
 
 private:
     ChatWindowStyle *m_chatStyle;
@@ -104,13 +117,14 @@ private:
     bool m_useCustomFont;
     QString m_fontFamily;
     int m_fontSize;
+    ShowPresenceMode m_showPresenceMode;
 
     QString appendScript(AppendMode mode);
     AppendMode appendMode(const AdiumThemeMessageInfo &message,
                           bool consecutive,
                           bool willAddMoreContentObjects,
                           bool replaceLastContent);
-    void appendMessage(QString&, AppendMode mode);
+    void appendMessage(QString &htmlMessage, const QString &script, AppendMode mode);
 
     QString replaceHeaderKeywords(QString htmlTemplate, const AdiumThemeHeaderInfo&);
     QString replaceContentKeywords(QString& htmlTemplate, const AdiumThemeContentInfo&);
