@@ -31,6 +31,8 @@ ChannelContactModel::ChannelContactModel(const Tp::TextChannelPtr &channel, QObj
 
 void ChannelContactModel::setTextChannel(const Tp::TextChannelPtr &channel)
 {
+    m_channel = channel;
+    
     //remove existing contacts in list
     beginRemoveRows(QModelIndex(), 0, m_contacts.size());
     m_contacts.clear();
@@ -45,8 +47,22 @@ void ChannelContactModel::setTextChannel(const Tp::TextChannelPtr &channel)
                                        Tp::Contacts,Tp::Channel::GroupMemberChangeDetails)),
             SLOT(onGroupMembersChanged(Tp::Contacts,Tp::Contacts,Tp::Contacts,
                                      Tp::Contacts,Tp::Channel::GroupMemberChangeDetails)));
+
+    connect(channel.data(),
+            SIGNAL(chatStateChanged(Tp::ContactPtr,Tp::ChannelChatState)),
+            SLOT(onChatStateChanged(Tp::ContactPtr,Tp::ChannelChatState)));
 }
 
+bool ChannelContactModel::containsNick(const QString& nick)
+{
+    Q_FOREACH(const Tp::ContactPtr &contact, m_contacts) {
+        if (contact->alias() == nick) {
+            return true;
+        }
+    }
+
+    return false;
+}
 
 int ChannelContactModel::rowCount(const QModelIndex &parent) const
 {
@@ -69,8 +85,14 @@ QVariant ChannelContactModel::data(const QModelIndex &index, int role) const
         return QVariant(m_contacts[row]->alias());
 
     case Qt::DecorationRole:
-        return KTp::Presence(m_contacts[row]->presence()).icon();
+    {
+        const Tp::ContactPtr contact = m_contacts[row];
+        if (m_channel->chatState(contact) == Tp::ChannelChatStateComposing) {
+            return KIcon(QLatin1String("document-edit"));
 
+        }
+        return KTp::Presence(contact->presence()).icon();
+    }
     default:
         return QVariant();
     }
@@ -154,3 +176,10 @@ void ChannelContactModel::removeContacts(const Tp::Contacts &contacts)
     }
 }
 
+void ChannelContactModel::onChatStateChanged(const Tp::ContactPtr &contact, Tp::ChannelChatState state)
+{
+    Q_UNUSED(state)
+    const QModelIndex index = createIndex(m_contacts.lastIndexOf(contact), 0);
+    dataChanged(index, index);
+}
+
