@@ -28,16 +28,18 @@
 #include <QCheckBox>
 #include <QDebug>
 #include <QHBoxLayout>
+#include <QToolButton>
 
 ChatSearchBar::ChatSearchBar(QWidget *parent)
     : QWidget(parent)
     , m_searchInput(new KLineEdit(this))
-    , m_closeButton(new KPushButton(this))
+    , m_closeButton(new QToolButton(this))
     , m_nextButton(new KPushButton(KIcon(QLatin1String("go-down-search")), i18nc("Next search result" ,"&Next"), this))
     , m_previousButton(new KPushButton(KIcon(QLatin1String("go-up-search")), i18nc("Previous search result" ,"&Previous"), this))
     , m_caseSensitive(false)
 {
     // close button setup
+    m_closeButton->setAutoRaise(true);
     m_closeButton->setIcon(KIcon(QLatin1String("dialog-close")));
     connect(m_closeButton, SIGNAL(clicked(bool)), this, SLOT(toggleView(bool)));
 
@@ -59,7 +61,10 @@ ChatSearchBar::ChatSearchBar(QWidget *parent)
     connect(m_searchInput, SIGNAL(textChanged(QString)), this, SLOT(textChanged(QString)));
 
     QHBoxLayout *layout = new QHBoxLayout(this);
+    layout->setContentsMargins(2, 0, 2, 0);
     layout->addWidget(m_closeButton);
+    layout->setAlignment(m_closeButton, Qt::AlignLeft | Qt::AlignTop);
+
     layout->addWidget(m_searchInput);
     layout->addWidget(m_nextButton);
     layout->addWidget(m_previousButton);
